﻿using System;
using System.Collections.Generic;
using System.IO;
using Ozeki.Common;
using Ozeki.Common.Logger;
using Ozeki.Network;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;

namespace DTMF_IVR_WinForm.Model
{
    /// <summary>
    /// Ez kezeli a softphonból jövő hívásokat, és megalkotja hozzájuk a menurendszert, valamint eltárolja az aktuális hívásokat.
    /// </summary>
    internal class IVREngine
    {
        /// <summary>
        /// The SoftPhone is provided by the VoIPSDK.
        /// </summary>
        private ISoftPhone softPhone;

        private IPhoneLine line;

        /// <summary>
        /// Collection that contains the current active calls.
        /// </summary>
        private List<CustomerCall> activeCalls;

        /// <summary>
        /// Logging request on GUI.
        /// </summary>
        public event EventHandler<GenericEventArgs<string>> NotifyAction;

        /// <summary>
        /// IVR menu system composer.
        /// </summary>
        private IVRMenuBuilder menuBuilder;


        /// <summary>
        /// Creates an instance of IVREngine and creates a phonline and registers it.
        /// </summary>
        /// <param name="account">A Sip PBX account.</param>
        public IVREngine(SIPAccount account)
        {
            activeCalls = new List<CustomerCall>();
            menuBuilder=new IVRMenuBuilder();
            Logger.Open(LogLevel.Notice);
            softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5060, 8000, 5060, Path.Combine(Environment.CurrentDirectory,"log.txt"));
            
            softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);
            line = softPhone.CreatePhoneLine(account,
                                      new NatConfiguration(NatTraversalMethod.None),
                                      TransportType.Udp);
            line.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(line_PhoneLineStateChanged);
            softPhone.RegisterPhoneLine(line);

        }

        /// <summary>
        /// Incoming call events that occur in SoftPhone.
        /// </summary>
        /// <param name="sender">The SoftPhone.</param>
        /// <param name="e">The incomming call.</param>
        void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            CustomerCall call = new CustomerCall(e.Item, menuBuilder.GetRootMenu());
            call.NotifyAction += new EventHandler<GenericEventArgs<string>>(call_NotifyAction);
            call.Closing += new EventHandler(call_Closing);
            call.PhoneCall.Accept();
            activeCalls.Add(call);
        }

        /// <summary>
        /// An active call ends, so it removes from the active call collection.
        /// </summary>
        /// <param name="sender">CustmerCall object</param>
        /// <param name="e"> Not used.</param>
        void call_Closing(object sender, EventArgs e)
        {
            CustomerCall customerCall = sender as CustomerCall;
            customerCall.NotifyAction -= call_NotifyAction;
            customerCall.Closing -= call_Closing;
            activeCalls.Remove(customerCall);
        }

        /// <summary>
        /// Message received for the GUI.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">message</param>
        void call_NotifyAction(object sender, GenericEventArgs<string> e)
        {
            OnNotifyAction(e.Item);
        }

        /// <summary>
        /// Indicates the state changing of the phoneline.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void line_PhoneLineStateChanged(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            switch (e.Item)
            {
                case PhoneLineState.RegistrationSucceeded:
                    OnNotifyAction(string.Format("Phoneline registration successed to {0}", line.SIPAccount.DomainServerHost));

                    break;
                default:
                    OnNotifyAction(string.Format("PhonelineStateChanged: {0}", e.Item));
                    break;
            }
        }

        /// <summary>
        /// Sends the current event message to the GUI.
        /// </summary>
        /// <param name="actionMessage">Message.</param>
        private void OnNotifyAction(string actionMessage)
        {
            if (NotifyAction != null)
                NotifyAction(this, new GenericEventArgs<string>(actionMessage));
        }
    }
}
